package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysDictionaryService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.bean.SysDictionary;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description : 字典管理服务
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-28 23:54
 * @Modify :
 **/
@RestController
@RequestMapping("/sys-dictionary")
@Api(description = "字典管理服务")
public class SysDictionaryController {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysConfigController.class);

    @Autowired
    public ISysDictionaryService sysDictionaryService;

    @PostMapping("/insert")
    @ApiOperation(value = "添加字典",notes = "添加字典")
    public ResultBean insertSysDictionary(SysDictionary sysDictionary) {
        try {
            // 登录用户
            SessionUser user = AuthUtil.getSessionUser();

            // 条件验证
            ValidatorBean.beginValid(sysDictionary)
                    .notNull("name",sysDictionary.getName())
                    .notNull("dictionaryCode",sysDictionary.getDictionaryCode())
                    .notNull("dictionaryValue",sysDictionary.getDictionaryValue())
                    .notNull("dictionarySort",sysDictionary.getDictionarySort());

            //新增初始化
            ConvertBean.modelInitialize(sysDictionary,user);

            sysDictionaryService.insertSysDictionary(sysDictionary);
            return ResultBean.success("添加成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping("/update")
    @ApiOperation(value = "修改字典",notes = "修改字典")
    public ResultBean updateSysDictionary(SysDictionary sysDictionary) {
        try {
            // 登录用户
            SessionUser user = AuthUtil.getSessionUser();

            // 条件验证
            ValidatorBean.beginValid(sysDictionary)
                    .notNull("name",sysDictionary.getName())
                    .notNull("dictionaryCode",sysDictionary.getDictionaryCode())
                    .notNull("dictionaryValue",sysDictionary.getDictionaryValue())
                    .notNull("dictionarySort",sysDictionary.getDictionarySort());

            //修改初始化
            ConvertBean.modelUpdate(sysDictionary,user);

            sysDictionaryService.updateSysDictionary(sysDictionary);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @DeleteMapping("/delete/{id}")
    @ApiOperation(value = "根据id删除字典",notes = "根据id删除字典")
    public ResultBean updateSysDictionary(@PathVariable("id") String id) {
        try {
            // 条件验证
            ValidatorBean.checkNotNull(id,"id不能为空");

            sysDictionaryService.deleteSysDictionaryById(id);
            return ResultBean.success("删除成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/list")
    @ApiOperation(value = "查询所有字典", notes = "查询所有字典")
    public ResultBean findSysDictionaryAll() {
        try {
            List<SysDictionary> sysDictionaryList = sysDictionaryService.listSysDictionary();
            return ResultBean.success("查询成功")
                            .setResultList(sysDictionaryList)
                            .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "根据id查询字典", notes = "根据id查询字典")
    public ResultBean getSysDictionaryById(@PathVariable("id") String id) {
        try {
            // 条件判断
            ValidatorBean.checkNotNull(id,"id不能为空");

            SysDictionary sysDictionary = sysDictionaryService.getSysDictionaryById(id);
            if (sysDictionary != null) {
                return ResultBean.success("查询成功").setResultObject(sysDictionary)
                        .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
            }else {
                return ResultBean.fail("字典不存在").setCode(ResourceEnumUtil.MESSAGE.EMPTY.getCode());
            }
        } catch (ImppBusiException busExcep) {
            busExcep.printStackTrace();
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            e.printStackTrace();
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/find/{code}")
    @ApiOperation(value = "根据父级code查询字典项", notes = "根据父级code查询字典项")
    public ResultBean findSysDictionaryByParentCode(@PathVariable("code") String code) {
        try {
            List<SysDictionary> sysDictionaryList = sysDictionaryService.findSysDictionaryByParentCode(code);
            return ResultBean.success("查询成功").setResultList(sysDictionaryList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping("/query")
    @ApiOperation(value = "字典复杂查询，分页，排序",notes = "字典复杂查询，分页，排序")
    public ResultBean querySysDictionaryByPager(SysDictionary sysDictionary, Pager pager) {
        try {
            ListPager sysDictionaryListPager = sysDictionaryService.querySysDictionaryByPager(sysDictionary,pager);
            return  ResultBean.success("查询成功")
                    .setListPager(sysDictionaryListPager)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping("/find/top")
    @ApiOperation(value = "查询顶级字典项",notes = "查询顶级字典项")
    public ResultBean findSysDictionaryByTop() {
        try {
            List<SysDictionary> sysDictionaryList = sysDictionaryService.findSysDictionaryByTop();
            return  ResultBean.success("查询成功")
                    .setResultList(sysDictionaryList)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }
}
