 package cn.estsh.i3plus.core.apiservice.controller.busi;

 import cn.estsh.i3plus.core.api.iservice.busi.ISysMenuService;
 import cn.estsh.i3plus.platform.common.convert.ConvertBean;
 import cn.estsh.i3plus.pojo.base.bean.ListPager;
 import cn.estsh.i3plus.pojo.base.common.Pager;
 import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
 import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
 import cn.estsh.i3plus.pojo.platform.bean.Organize;
 import cn.estsh.i3plus.pojo.platform.bean.SysMenu;
 import cn.estsh.impp.framework.boot.auth.AuthUtil;
 import cn.estsh.impp.framework.boot.exception.ImppBusiException;
 import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
 import cn.estsh.impp.framework.boot.util.ResultBean;
 import cn.estsh.impp.framework.boot.util.ValidatorBean;
 import io.swagger.annotations.Api;
 import io.swagger.annotations.ApiOperation;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.springframework.beans.factory.annotation.Autowired;
 import org.springframework.web.bind.annotation.*;

 import java.util.List;

/**
 * @Description : 系统功能对外接口
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-22 15:18:14.982
 * @Modify :
 **/
@RestController
@RequestMapping("/sys-menu")
@Api(description = "系统功能服务")
public class SysMenuController {

    public static final Logger LOGGER = LoggerFactory.getLogger(SysMenuController.class);

    @Autowired
    private ISysMenuService sysMenuService;

    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增系统功能", notes = "系统功能")
    public ResultBean insertSysMenu(SysMenu menu) {
        try {
            //条件验证
            ValidatorBean.beginValid(menu)
                    .notNull("name", menu.getName())
                    .notNull("menuTypeId", menu.getMenuTypeId());

            //新增初始化
            ConvertBean.modelInitialize(menu, AuthUtil.getSessionUser());
            if(menu.getParentId() == null || menu.getParentId().intValue() == 0){
                menu.setParentId(CommonEnumUtil.PARENT.DEFAULT.getValue());
            }
            menu.setMenuStatus(CommonEnumUtil.DATA_STATUS.ENABLE.getValue());

            menu = sysMenuService.insertSysMenu(menu);
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).setResultObject(menu);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow()).build();
        } catch (Exception e) {
            return ResultBean.fail(e.getMessage()).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping(value = "/update")
    @ApiOperation(value = "修改系统功能", notes = "系统功能")
    public ResultBean updateSysMenu(SysMenu sysMenu) {
        try {
            // 数据校验
            ValidatorBean.beginValid(sysMenu)
                    .notNull("id", sysMenu.getId())
                    .notNull("menuTypeId", sysMenu.getMenuTypeId())
                    .notNull("name", sysMenu.getName());

            ConvertBean.modelUpdate(sysMenu, AuthUtil.getSessionUser());

            sysMenuService.updateSysMenu(sysMenu);
            return ResultBean.success("修改成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow()).build();
        } catch (Exception e) {
            return ResultBean.fail(e.getMessage()).setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @DeleteMapping(value = "/delete/{id}")
    @ApiOperation(value = "删除系统功能", notes = "删除系统功能")
    public ResultBean deleteSysMenu(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "系统功能id 不能为空");

            sysMenuService.deleteSysMenuById(id);
            return ResultBean.success().setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/list")
    @ApiOperation(value = "查询所有系统功能", notes = "查询所有系统功能")
    public ResultBean findAll() {
        try {
            List<SysMenu> list = sysMenuService.findSysMenuAll();
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value="/query")
    @ApiOperation(value="查询所有权限",notes="查询所有权限")
    public ResultBean queryPosition(SysMenu menu, Pager pager){
        try{
            ListPager<SysMenu> listPager = sysMenuService.querySysMenu(menu, pager);

            return ResultBean.success("操作成功").setListPager(listPager)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/get/{id}")
    @ApiOperation(value = "查询系统功能", notes = "查询根据ID查询系统功能")
    public ResultBean getById(@PathVariable("id") String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "功能id 不能为空");

            SysMenu sysMenu = sysMenuService.getSysMenuById(id);

            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(sysMenu);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }

    }

    @PutMapping(value = "/status")
    @ApiOperation(value = "修改系统功能状态", notes = "根据ID修改系统功能状态")
    public ResultBean updateStatus(String id,int status) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "功能id 不能为空");
            ValidatorBean.checkNotZero(status, "功能状态不能为空");

            sysMenuService.updateSysMenuStatus(id,status,AuthUtil.getSessionUser());
            return new ResultBean(true);
        } catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode());
        }
    }

    @PutMapping(value = "/batch-status")
    @ApiOperation(value = "批量修改系统功能状态", notes = "根据ID修改系统功能状态")
    public ResultBean updateBatchStatus(String[] ids,int status) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(ids, "功能id 不能为空");
            ValidatorBean.checkNotZero(status, "功能状态不能为空");

            sysMenuService.updateSysMenuStatusByIds(ids,status,AuthUtil.getSessionUser());

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode());
        }
    }




}
