package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ICoreTreeService;
import cn.estsh.i3plus.pojo.platform.bean.*;
import cn.estsh.i3plus.pojo.platform.repository.DepartmentRepository;
import cn.estsh.i3plus.pojo.platform.repository.OrganizeRepository;
import cn.estsh.i3plus.pojo.platform.repository.PositionRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysMenuRepository;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import javafx.scene.layout.VBox;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : Core 树功能实现
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-24 11:17
 * @Modify :
 **/
//TODO for -> 传统for
@Service
public class CoreTreeServiceImpl implements ICoreTreeService {

    public static final Logger LOGGER = LoggerFactory.getLogger(CoreTreeServiceImpl.class);

    @Autowired
    private DepartmentRepository departmentRDao;

    @Autowired
    private PositionRepository positionRDao;

    @Autowired
    private OrganizeRepository organizeRDao;

    @Autowired
    private SysMenuRepository menuRDao;

    @Override
    public List<Position> findPositionTreeByParentId(long parentId) {
        List<Position> result = positionRDao.findByProperty("parentId", parentId);

        // 循环设置子集
        if(result != null && result.size() > 0){
            result.stream().forEach(position -> findPositionChildrenTreePack(position,0));
        }

        return result;
    }

    @Override
    public Position getPositionTreeByParentId(long id) {
        // 查找数据
        Position position = positionRDao.getById(id);
        // 数据是否真实存在
        ValidatorBean.checkNotNull(position,"不存在的岗位信息");
        // 递归查询子集
        findPositionChildrenTreePack(position, 0);

        return position;
    }

    @Override
    public void findPositionChildrenTreePack(Position parent, int step) {
        ++step;
        List<Position> childList = positionRDao.findByProperty("parentId", parent.getId());

        if (childList.size() > 0) {
            //说明有子集
            for (Position child : childList) {
                findPositionChildrenTreePack(child, step);
            }
        }

        LOGGER.info("【{}】包含子集：{},步长：{}", parent.getName(), childList.size(), step);
        parent.setChildList(childList);
    }

    @Override
    public List<Department> findDepartmentTreeByParentId(long parentId) {
        List<Department> result = departmentRDao.findByProperty("parentId", parentId);

        // 循环设置子集
        if(result != null && result.size() > 0){
            result.stream().forEach(department -> findDepartmentChildrenTreePack(department,0));
        }

        return result;
    }

    @Override
    public Department getDepartmentTreeByParentId(long id) {
        // 查找数据
        Department department = departmentRDao.getById(id);
        // 数据是否真实存在
        ValidatorBean.checkNotNull(department,"不存在的部门信息");
        // 递归查询子集
        findDepartmentChildrenTreePack(department, 0);

        return department;
    }

    @Override
    public void findDepartmentChildrenTreePack(Department parent, int step) {
        ++step;
        List<Department> childList = departmentRDao.findByProperty("parentId", parent.getId());

        if (childList.size() > 0) {
            //说明有子集
            for (Department child : childList) {
                findDepartmentChildrenTreePack(child, step);
            }
        }

        LOGGER.info("【{}】包含子集：{},步长：{}", parent.getName(), childList.size(), step);
        parent.setChildList(childList);
    }

    @Override
    public List<Organize> findOrganizeTreeByParentId(long parentId) {
        List<Organize> result = organizeRDao.findByProperty("parentId", parentId);

        // 循环设置子集
        if(result != null && result.size() > 0){
            result.stream().forEach(organize -> findOrganizeChildrenTreePack(organize,0));
        }

        return result;
    }

    @Override
    public Organize getOrganizeTreeByParentId(long id) {
        // 查找数据
        Organize organize = organizeRDao.getById(id);
        // 数据是否真实存在
        ValidatorBean.checkNotNull(organize,"不存在的组织信息");
        // 递归查询子集
        findOrganizeChildrenTreePack(organize, 0);

        return organize;
    }

    @Override
    public void findOrganizeChildrenTreePack(Organize parent, int step) {
        ++step;
        List<Organize> childList = organizeRDao.findByProperty("parentId", parent.getId());

        if (childList.size() > 0) {  //说明有子集
            for (Organize child : childList) {
                findOrganizeChildrenTreePack(child, step);
            }
        }

        LOGGER.info("【{}】包含子集：{},步长：{}", parent.getName(), childList.size(), step);
        parent.setChildList(childList);
    }

    @Override
    public List<SysMenu> findSysMenuTreeByParentId(long parentId) {
        List<SysMenu> result = menuRDao.findByProperty("parentId", parentId);

        // 循环设置子集
        if(result != null && result.size() > 0){
            result.stream().forEach(menu -> findSysMenuChildrenTreePack(menu,0));
        }

        return result;
    }

    @Override
    public SysMenu getSysMenuTreeByParentId(long id) {
        // 查找数据
        SysMenu menu = menuRDao.getById(id);
        // 数据是否真实存在
        ValidatorBean.checkNotNull(menu,"不存在的组织信息");
        // 递归查询子集
        findSysMenuChildrenTreePack(menu, 0);

        return menu;
    }

    @Override
    public void findSysMenuChildrenTreePack(SysMenu parent, int step) {
        ++step;
        List<SysMenu> childList = menuRDao.findByProperty("parentId", parent.getId());

        if (childList.size() > 0) {  //说明有子集
            for (SysMenu child : childList) {
                findSysMenuChildrenTreePack(child, step);
            }
        }

        LOGGER.info("【{}】包含子集：{},步长：{}", parent.getName(), childList.size(), step);
        parent.setChildList(childList);
    }
}
