package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.IOrganizeService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.Organize;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.i3plus.pojo.platform.repository.OrganizeRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 组织业务接口实现 
 * @Reference   :
 * @Author      : wei.peng
 * @Date        : 2018-10-22 16:58:42.957
 * @Modify      :
 **/
@Service
public class OrganizeServiceImpl implements IOrganizeService {

    public static final Logger LOGGER = LoggerFactory.getLogger(OrganizeServiceImpl.class);

    @Autowired
    private OrganizeRepository organizeRDao;

    @Override
    public void updateOrganize(Organize organize) {
        LOGGER.info("组织 ORGANIZE :{}", organize);

        if(organize.getParentId() != null){
            Organize org = getOrganizeById(organize.getParentId().toString());
            ValidatorBean.checkNotNull(org);

            organize.setRedParentName(org.getName());
        }

        organizeRDao.update(organize);
    }

    @Override
    public void updateOrganizeStatus(String id, int status, SessionUser user) {
        LOGGER.info("组织 ORGANIZE  id:{} status:{} user:{}", id,status,user);
        Organize organize = organizeRDao.getById(Long.parseLong(id));
        organize.setOrganizeStatusId(status);
        organize.setModifyUser(user.getUserName());
        organizeRDao.update(organize);
    }

    @Override
    public void deleteOrganizeById(String id) {
        LOGGER.info("组织 ORGANIZE Key:{}", id);
        long count = organizeRDao.findByPropertyCount("parentId", Long.parseLong(id));
        if (count >= 1) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.IMPP.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION_DATA_NOT_EXIT.getCode())
                    .setErrorDetail("该数据有子集")
                    .setErrorSolution("请先删除子集在操作")
                    .build();
        }
        organizeRDao.deleteById(Long.parseLong(id));
    }

    @Override
    public void deleteOrganizeByIds(String[] ids) {
        long positionCount = organizeRDao.findByHqlWhereCount(CoreHqlPack.packHqlIds("parentId",ids));
        if (positionCount >= 1) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("存在子节点无法删除！")
                    .setErrorSolution("请重新操作")
                    .build();
        }else {
            Long[] array = new Long[ids.length];
            for (int i = 0; i < ids.length; i++) {
                array[i] = Long.parseLong(ids[i]);
            }
            organizeRDao.deleteByIds(array);
        }
    }

    @Override
    public Organize insertOrganize(Organize organize) {
        LOGGER.info("组织 ORGANIZE :{}", organize);

        if(organize.getParentId() != null && organize.getParentId().intValue() > 0){
            Organize org = getOrganizeById(organize.getParentId().toString());
            ValidatorBean.checkNotNull(org);

            organize.setRedParentName(org.getName());
        }
        return organizeRDao.insert(organize);
    }

    @Override
    public List<Organize> listOrganize() {
        LOGGER.info("组织 ORGANIZE find All");
        return organizeRDao.findAll();
    }

    @Override
    public ListPager<Organize> queryOrganize(Organize organize, Pager pager) {
        LOGGER.info("组织信息 Organize find Organize page");

        if(organize == null){
            //不传入实体对象，查询所有
            int count = organizeRDao.listCount();
            pager = PagerHelper.getPager(pager,count);
            return new ListPager(organizeRDao.listPager(pager),pager);
        }else {
            //生成hql查询语句
            String hqlPack = CoreHqlPack.packHqlOrganize(organize);
            pager = PagerHelper.getPager(pager,organizeRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(organizeRDao.findByHqlWherePage(hqlPack + organize.orderBy(), pager),pager);
        }
    }

    @Override
    public Organize getOrganizeById(String id) {
        LOGGER.info("组织 ORGANIZE find id:{}", id);
        return organizeRDao.getById(Long.parseLong(id));
    }
}
