package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysMessageService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysMessage;
import cn.estsh.i3plus.pojo.platform.bean.SysUser;
import cn.estsh.i3plus.pojo.platform.repository.SysMessageRepository;
import cn.estsh.i3plus.pojo.platform.repository.SysUserRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 消息服务接口实现
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-25 18:02
 * @Modify :
 **/
@Service
public class SysMessageService implements ISysMessageService {
    public static final Logger LOGGER = LoggerFactory.getLogger(SysMessageService.class);

    @Autowired
    public SysMessageRepository sysMessageRDao;

    @Autowired
    public SysUserRepository sysUserRDao;

    @Override
    public SysMessage insertSysMessage(SysMessage sysMessage) {
        // 查询接收人名称
        if (sysMessage.getMessageRecipientId() != null  && sysMessage.getMessageRecipientId() > 0) {
            LOGGER.info("用户信息 SYSUSER id:{}", sysMessage.getMessageRecipientId());
            SysUser acceptUser = sysUserRDao.getById(sysMessage.getMessageRecipientId());
            ValidatorBean.checkNotNull(acceptUser);

            sysMessage.setRedMessageRecipientName(acceptUser.getName());
        }

        LOGGER.info("消息 SYS_MESSAGE :{}",sysMessage);
        return sysMessageRDao.insert(sysMessage);
    }

    @Override
    public void deleteSysMessageById(String id) {
        LOGGER.info("消息 SYS_MESSAGE id:{}",id);
        sysMessageRDao.deleteById(Long.parseLong(id));

    }

    @Override
    public void updateSysMessage(SysMessage sysMessage) {
        // 查询接收人名称
        if (sysMessage.getMessageRecipientId() != null  && sysMessage.getMessageRecipientId() > 0) {
            LOGGER.info("用户信息 SYSUSER id:{}", sysMessage.getMessageRecipientId());
            SysUser acceptUser = sysUserRDao.getById(sysMessage.getMessageRecipientId());
            ValidatorBean.checkNotNull(acceptUser);

            sysMessage.setRedMessageRecipientName(acceptUser.getName());
        }

        LOGGER.info("消息 SYS_MESSAGE :{}",sysMessage);
        sysMessageRDao.update(sysMessage);
    }

    @Override
    public List<SysMessage> listSysMessage() {
        LOGGER.info("消息 SYS_MESSAGE list");
        return sysMessageRDao.list();
    }

    @Override
    public SysMessage getSysMessageById(String id) {
        LOGGER.info("消息 SYS_MESSAGE id:{}",id);
        return sysMessageRDao.getById(Long.parseLong(id));
    }

    @Override
    public ListPager querySysMessageByPager(SysMessage sysMessage, Pager pager) {
        LOGGER.info("消息 SYS_MESSAGE SysMessage:{}，Pager:{}",sysMessage,pager);
        if(sysMessage == null) {
            pager = PagerHelper.getPager(pager, sysMessageRDao.listCount());
            return new ListPager(sysMessageRDao.listPager(pager),pager);
        }else {
            String hqlPack = CoreHqlPack.packHqlSysMessage(sysMessage);
            pager = PagerHelper.getPager(pager, sysMessageRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(sysMessageRDao.findByHqlWherePage(hqlPack + sysMessage.orderBy(),pager),pager);
        }
    }

    @Override
    public void updateSysMessageStatusById(String id, Integer status) {
        LOGGER.info("消息 SYS_MESSAGE id:{}，status:{}",id,status);
        SysMessage sysMessage = sysMessageRDao.getById(Long.parseLong(id));
        sysMessage.setMessageStatusId(status);

        sysMessageRDao.update(sysMessage);
    }

    @Override
    public void deleteSysMessageByIds(String[] ids) {
        LOGGER.info("硬件类型 TOOL_TYPE ids:{}", ids.toString());
        long[] array = new long[ids.length];
        for (int i = 0; i < ids.length; i++) {
            array[i] = Long.parseLong(ids[i]);
        }

        sysMessageRDao.deleteByIds(array);
    }

    @Override
    public void updateSysMessageStatusByIds(String[] ids,Integer status) {
        LOGGER.info("硬件类型 TOOL_TYPE ids:{}", ids.toString());
        long[] array = new long[ids.length];
        for (int i = 0; i < ids.length; i++) {
            array[i] = Long.parseLong(ids[i]);
        }
        // TODO 批量修改消息状态
//        sysMessageRDao.
    }
}
