package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.IToolService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.Tool;
import cn.estsh.i3plus.pojo.platform.bean.ToolType;
import cn.estsh.i3plus.pojo.platform.repository.ToolRepository;
import cn.estsh.i3plus.pojo.platform.repository.ToolTypeRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 硬件服务接口实现
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-25 20:29
 * @Modify :
 **/
@Service
public class ToolService implements IToolService {
    public static final Logger LOGGER = LoggerFactory.getLogger(ToolService.class);

    @Autowired
    private ToolRepository toolRDao;

    @Autowired
    private ToolTypeRepository toolTypeRDao;

    @Override
    public void insertTool(Tool tool) {
        // 查询硬件类型名称
        LOGGER.info("硬件类型 TOOL_TYPE :{}", tool.getToolTypeId());
        ToolType toolType = toolTypeRDao.getById(tool.getToolTypeId());
        ValidatorBean.checkNotNull(toolType);

        tool.setRedToolTypeName(toolType.getName());

        LOGGER.info("硬件 TOOL tool:{}",tool);
        toolRDao.insert(tool);
    }

    @Override
    public void deleteToolById(String id) {
        LOGGER.info("硬件 TOOL id:{}",id);
        toolRDao.deleteById(Long.parseLong(id));
    }

    @Override
    public void updateTool(Tool tool) {
        // 查询硬件类型名称
        LOGGER.info("硬件类型 TOOL_TYPE :{}", tool.getToolTypeId());
        ToolType toolType = toolTypeRDao.getById(tool.getToolTypeId());
        ValidatorBean.checkNotNull(toolType);

        tool.setRedToolTypeName(toolType.getName());

        LOGGER.info("硬件 TOOL :{}",tool);
        toolRDao.update(tool);
    }

    @Override
    public List<Tool> listTool() {
        LOGGER.info("硬件 TOOL list");
        return toolRDao.list();
    }

    @Override
    public Tool getToolById(String id) {
        LOGGER.info("硬件 TOOL id:{}",id);
        return toolRDao.getById(Long.parseLong(id));
    }

    @Override
    public ListPager queryToolByPager(Tool tool, Pager pager) {
        LOGGER.info("硬件 TOOL Tool:{}，Pager:{}",tool,pager);
        if(tool == null) {
            pager = PagerHelper.getPager(pager, toolRDao.listCount());
            return new ListPager(toolRDao.listPager(pager),pager);
        }else {
            String hqlPack = CoreHqlPack.packHqlTool(tool);
            pager = PagerHelper.getPager(pager, toolRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(toolRDao.findByHqlWherePage(hqlPack + tool.orderBy(),pager),pager);
        }
    }

    @Override
    public void updateStatusEnable(String id) {
        LOGGER.info("硬件 TOOL id:{}",id);
        toolRDao.updateByProperties("id",Long.parseLong(id),
                    "toolStatusId", CommonEnumUtil.DATA_STATUS.ENABLE.getValue());
    }

    @Override
    public void updateStatusDisable(String id) {
        LOGGER.info("硬件 TOOL id:{}",id);
        toolRDao.updateByProperties("id",Long.parseLong(id),
                "toolStatusId", CommonEnumUtil.DATA_STATUS.DISABLE.getValue());
    }
}
