package cn.estsh.i3plus.core.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.core.api.iservice.busi.IToolTypeService;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.ToolType;
import cn.estsh.i3plus.pojo.platform.repository.ToolRepository;
import cn.estsh.i3plus.pojo.platform.repository.ToolTypeRepository;
import cn.estsh.i3plus.pojo.platform.sqlpack.CoreHqlPack;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @Description : 硬件类型接口实现
 * @Reference :
 * @Author : yunhao
 * @Date : 2018-10-28 12:35
 * @Modify :
 **/
@Service
public class ToolTypeService implements IToolTypeService {
    public static final Logger LOGGER = LoggerFactory.getLogger(ToolTypeService.class);

    @Autowired
    private ToolTypeRepository toolTypeRDao;
    @Autowired
    private ToolRepository toolRDao;

    @Override
    public void insertToolType(ToolType toolType) {
        LOGGER.info("硬件类型 TOOL_TYPE toolType:{}",toolType);
        toolTypeRDao.insert(toolType);
    }

    @Override
    public void deleteToolTypeById(String id) {
        LOGGER.info("硬件类型 TOOL_TYPE id:{}",id);
        long count = toolRDao.findByPropertyCount("toolTypeId",id);
        if(count >= 1){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("存在关联硬件无法删除！")
                    .setErrorSolution("请重新操作")
                    .build();
        }

        toolTypeRDao.deleteById(Long.parseLong(id));
    }

    @Override
    public void updateToolType(ToolType toolType) {
        LOGGER.info("硬件类型 TOOL_TYPE toolType:{}",toolType);
        toolTypeRDao.update(toolType);
        // TODO 修改硬件对象中冗余的硬件类型名称
    }

    @Override
    public List<ToolType> listToolType() {
        LOGGER.info("硬件类型 TOOL_TYPE list");
        return toolTypeRDao.list();
    }

    @Override
    public ToolType getToolTypeById(String id) {
        LOGGER.info("硬件类型 TOOL_TYPE id:{}",id);
        return toolTypeRDao.getById(Long.parseLong(id));
    }

    @Override
    public ListPager queryToolTypeByPager(ToolType toolType, Pager pager) {
        LOGGER.info("硬件类型 TOOL_TYPE ToolType:{}，Pager:{}",toolType,pager);
        if(toolType == null) {
            pager = PagerHelper.getPager(pager, toolTypeRDao.listCount());
            return new ListPager(toolTypeRDao.listPager(pager),pager);
        }else {
            String hqlPack = CoreHqlPack.packHqlToolType(toolType);
            pager = PagerHelper.getPager(pager, toolTypeRDao.findByHqlWhereCount(hqlPack));
            return new ListPager(toolTypeRDao.findByHqlWherePage(hqlPack + toolType.orderBy(),pager),pager);
        }
    }

    @Override
    public void deleteToolTypeByIds(String[] ids) {
        LOGGER.info("硬件类型 TOOL_TYPE ids:{}", ids.toString());
        long count = toolRDao.findByHqlWhereCount(CoreHqlPack.packHqlIds("toolTypeId",ids));
        if(count >= 1){
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("存在关联硬件无法删除！")
                    .setErrorSolution("请重新操作")
                    .build();
        }else {
            Long[] array = new Long[ids.length];
            for (int i = 0; i < ids.length; i++) {
                array[i] = Long.parseLong(ids[i]);
            }
            toolTypeRDao.deleteByIds(array);
        }
    }
}
